@@ -72,10 +72,10 @@ sub new {
         unless ( $self->{type} ) {
 
             # Use local conf to get configStorage and localStorage
-            my $localconf =
+            my $localConfAccessPrms =
               $self->getLocalConf( CONFSECTION, $self->{confFile}, 0 );
-            if ( defined $localconf ) {
-                %$self = ( %$self, %$localconf );
+            if ( defined $localConfAccessPrms ) {
+                %$self = ( %$self, %$localConfAccessPrms );
             }
         }
         unless ( $self->{type} ) {
@@ -149,7 +149,8 @@ sub saveConf {
         eval { Lemonldap::NG::Handler::Main->reload() };
     }
 
-    return ( $self->unlock() ? $tmp : UNKNOWN_ERROR );
+    $self->unlock();
+    return $tmp;
 }
 
 ## @method hashRef getConf(hashRef args)
