@@ -120,10 +120,13 @@ sub init {
             my $lang = $1 || 'all';
             $self->trOver->{$lang}->{"PE$2"} = $self->localConfig->{$k};
         }
-        elsif ( $k =~ /msg_(?:(\w+?)_)?(\w+)$/ ) {
+        elsif ( $k =~ /msg_(?:([a-z][a-z](?:_[A-Z][A-Z])?)_)?(\w+)$/ ) {
             my $lang = $1 || 'all';
             $self->trOver->{$lang}->{$2} = $self->localConfig->{$k};
         }
+        else {
+            $self->conf->{$k} = $self->localConfig->{$k};
+        }
     }
     $self->trOver( JSON::to_json( $self->trOver ) );
 
@@ -280,7 +283,7 @@ sub reloadConf {
         [ $self->{templateDir}, $self->conf->{templateDir} . '/bootstrap' ] );
 
     $self->{staticPrefix} = $self->conf->{staticPrefix} || '/static';
-    $self->{languages}    = $self->conf->{languages}    || '/';
+    $self->{languages}    = $self->conf->{languages}    || 'en';
 
     # Initialize session DBs
     unless ( $self->conf->{globalStorage} ) {
